@@ -6,11 +6,9 @@ from allauth.socialaccount.providers.oauth2.views import (
     OAuth2LoginView,
 )
 
-from .provider import BitbucketOAuth2Provider
-
 
 class BitbucketOAuth2Adapter(OAuth2Adapter):
-    provider_id = BitbucketOAuth2Provider.id
+    provider_id = "bitbucket_oauth2"
     access_token_url = "https://bitbucket.org/site/oauth2/access_token"
     authorize_url = "https://bitbucket.org/site/oauth2/authorize"
     profile_url = "https://api.bitbucket.org/2.0/user"
@@ -41,9 +39,8 @@ class BitbucketOAuth2Adapter(OAuth2Adapter):
             primary_emails = [e for e in emails if e.get("is_primary", False)]
             email = primary_emails[0].get("email")
         except (IndexError, TypeError, KeyError):
-            return ""
-        finally:
-            return email
+            pass
+        return email
 
 
 oauth_login = OAuth2LoginView.adapter_view(BitbucketOAuth2Adapter)
