@@ -48,3 +48,6 @@ class CleverOAuth2Tests(OAuth2TestsMixin, TestCase):
                 }""",
             ),
         ]
+
+    def get_expected_to_str(self):
+        return "Clever"
