@@ -7,8 +7,6 @@ from allauth.socialaccount.providers.oauth.views import (
     OAuthLoginView,
 )
 
-from .provider import FiveHundredPxProvider
-
 
 API_BASE = "https://api.500px.com/v1"
 
@@ -25,7 +23,7 @@ class FiveHundredPxAPI(OAuth):
 
 
 class FiveHundredPxOAuthAdapter(OAuthAdapter):
-    provider_id = FiveHundredPxProvider.id
+    provider_id = "500px"
     request_token_url = API_BASE + "/oauth/request_token"
     access_token_url = API_BASE + "/oauth/access_token"
     authorize_url = API_BASE + "/oauth/authorize"
